package com.supervision.police.service.impl;

import com.alibaba.druid.util.StringUtils;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.supervision.common.domain.R;
import com.supervision.neo4j.domain.CaseNode;
import com.supervision.neo4j.domain.Rel;
import com.supervision.neo4j.service.Neo4jService;
import com.supervision.police.domain.*;
import com.supervision.police.mapper.ModelRecordTypeMapper;
import com.supervision.police.mapper.NoteRecordSplitMapper;
import com.supervision.police.service.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.ai.chat.ChatResponse;
import org.springframework.ai.chat.messages.UserMessage;
import org.springframework.ai.chat.prompt.Prompt;
import org.springframework.ai.ollama.OllamaChatClient;
import org.springframework.stereotype.Service;
import org.springframework.util.StopWatch;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;

@Slf4j
@Service
@RequiredArgsConstructor
public class ModelRecordTypeServiceImpl extends ServiceImpl<ModelRecordTypeMapper, ModelRecordType> implements ModelRecordTypeService {

    private final ModelRecordTypeMapper modelRecordTypeMapper;

    private final NoteRecordSplitMapper noteRecordSplitMapper;

    private final NotePromptService notePromptService;

    private final TripleInfoService tripleInfoService;

    private final Neo4jService neo4jService;

    private final OllamaChatClient chatClient;

    private final CaseTaskRecordService caseTaskRecordService;

    private final ExtractTripleInfoService extractTripleInfo;

    @Override
    public List<ModelRecordType> queryType(String name, Integer page, Integer size) {
        List<ModelRecordType> list = modelRecordTypeMapper.selectByName(name);

        for (ModelRecordType modelRecordType : list) {
            //笔录内容
            List<NoteRecordSplit> noteRecords = noteRecordSplitMapper.selectByRecordType(modelRecordType.getRecordType());
            modelRecordType.setRecords(noteRecords);
            //提示词
            List<NotePrompt> prompts = notePromptService.lambdaQuery().eq(NotePrompt::getTypeId, modelRecordType.getId()).list();
            modelRecordType.setPrompts(prompts);
        }
        return list;
    }

    @Override
    public ModelRecordType queryByName(String content) {
        Wrapper<ModelRecordType> wrapper = new QueryWrapper<ModelRecordType>().eq("record_type", content);
        return modelRecordTypeMapper.selectOne(wrapper);
    }

    @Override
    public R<?> saveType(ModelRecordType type) {
        String id = type.getId();
        int i = 0;
        if (StringUtils.isEmpty(id)) {
            //新增
            i = modelRecordTypeMapper.insert(type);
        } else {
            i = modelRecordTypeMapper.updateById(type);
        }
        if (i > 0) {
            return R.ok("保存成功");
        } else {
            return R.fail("保存失败");
        }
    }

    @Override
    public R<?> addOrUpdPrompt(NotePrompt prompt) {
        int i = 0;
        boolean save;
        if (StringUtils.isEmpty(prompt.getId())) {
            save = notePromptService.save(prompt);
        } else {
            save = notePromptService.updateById(prompt);
        }
        if (save) {
            return R.ok("保存成功");
        } else {
            return R.fail("保存失败");
        }
    }

    @Override
    public R<?> delPrompt(NotePrompt prompt) {
        String id = prompt.getId();
        boolean removeById = notePromptService.removeById(id);
        if (removeById) {
            return R.ok("删除成功");
        } else {
            return R.fail("删除失败");
        }
    }


    @Override
    public List<TripleInfo> getThreeInfo(String caseId, String name, String recordId) {
        //boolean taskStatus = taskExtractStatusCheck(caseId, recordId);
        // 如果校验结果为false,则说明需要进行提取三元组操作
        //if (!taskStatus) {
        //    extractTripleInfo.extractTripleInfo(caseId, name, recordId);
        //}
        // 这里进行查询
        return tripleInfoService.lambdaQuery().eq(TripleInfo::getRecordId, recordId).list();
    }

    /**
     * 提取任务校验,校验是否已经存在相关的人物,如果存在相关的任务,就不再继续执行了,直接告诉任务正在执行中
     */
    private boolean taskExtractStatusCheck(String caseId, String recordId) {
        // 首先查询是否存在任务,如果不存在,就新建
        Optional<CaseTaskRecord> caseTaskRecordOpt = caseTaskRecordService.lambdaQuery().eq(CaseTaskRecord::getCaseId, caseId).eq(CaseTaskRecord::getRecordId, recordId).oneOpt();
        if (caseTaskRecordOpt.isEmpty()) {
            CaseTaskRecord newCaseTaskRecord = new CaseTaskRecord();
            newCaseTaskRecord.setCaseId(caseId);
            newCaseTaskRecord.setRecordId(recordId);
            newCaseTaskRecord.setStatus(1);
            newCaseTaskRecord.setSubmitTime(LocalDateTime.now());
            caseTaskRecordService.save(newCaseTaskRecord);
            return false;
        } else {

            // 如果存在,则校验时间是否已经超过1天,如果超过了1天还没有执行完毕,就重新提交这个任务
            CaseTaskRecord caseTaskRecord = caseTaskRecordOpt.get();
            if (caseTaskRecordOpt.get().getStatus() == 1 && LocalDateTime.now().isAfter(caseTaskRecord.getSubmitTime().plusDays(1))) {
                // 如果已经超过1天,则重新提交任务
                caseTaskRecord.setStatus(1);
                caseTaskRecord.setSubmitTime(LocalDateTime.now());
                caseTaskRecordService.updateById(caseTaskRecord);
                return false;
            } else if (caseTaskRecordOpt.get().getStatus() == 2) {
                return true;
            } else if (caseTaskRecordOpt.get().getStatus() == 3) {
                caseTaskRecord.setStatus(1);
                caseTaskRecord.setSubmitTime(LocalDateTime.now());
                caseTaskRecordService.updateById(caseTaskRecord);
                return false;
            } else {
                // 如果没有超过1天,则返回正在执行中
                throw new RuntimeException("任务正在执行中");

            }
        }
    }

    @Override
    public void testExtractThreeInfo() {
        String prompt = """
                请从
                ---
                办案警官问：你是如何想到这个方法骗钱的? 裴金禄回答：因为我是在中卫市沙坡头区中兰铁路上做监理人员，我这样说的话别人会更容易相信。
                ---
                这段话中按(主体类型:行为人姓名;关系:伪造;客体类型:合同名称)提取特定的三元组信息。
                如果未提取到或关系不匹配或客体类型不匹配,则返回空的json即可。
                如果可以提取到三元组关系,则返回格式为{"主体":"主体名称","关系":"关系名称","客体":"客体名称"}。
                例如‘办案警官问:描述一下事情的经过。行为人小明答:我做了一份假的租车合同,然后骗小刚签了这个合同’，该示例存在给定的三元组关系.则结果为:{"主体":"小明","关系":"伪造","客体":"租车合同"}。
                例如‘办案警官问:办案警官问：讲一下那个软件的名称？小明回答：现在叫畅聊。‘该示例不存在给定的三元组关系,则结果为:{}
                请保证提取准确且完整，主体和客体要具体明确，不能是宽泛的概念。提取的客体要必须属于客体类型!

                以只返回JSON数据
                """;
        // 抽取的客体名称一定属于给定的客体类型分类,且抽取的关系完全等于给定的关系。
        Prompt ask = new Prompt(new UserMessage(prompt));
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        log.info("开始分析:");
        ChatResponse call = chatClient.call(ask);
        stopWatch.stop();
        log.info("耗时:{}", stopWatch.getTotalTimeSeconds());
        String content = call.getResult().getOutput().getContent();
        log.info("分析的结果是:{}", content);

        String oldPrompt = """
                请分析以下内容中所有规定的三元组信息并补充完整，
                要求：1.返回式：
                {result：
                   [
                       {startNodeType:'LawActor',entity:'',endNodeType:'FictionalOrgan',property:'',value:'冒充' },
                       {startNodeType:'LawActor',entity:'',endNodeType:'Seal',property:'',value:'伪造'},
                       {startNodeType:'LawActor',entity:'',endNodeType:'BusinessLicense',property:'',value:'伪造'}
                   ]
                }。
                2.必须考虑上下文语境分析。
                3.对话中不包含此类型三元组则返回{ "result": [] }。
                例子1：办案警官问：你为了骗取更多的钱都做了哪些准备？裴金禄回答：我刚开始我就是自己想了一些关于骗钱的点子，后面为了更不容易让别人识破我为了更佳逼真，我就从网上随便搜了一家租赁公司，我就搜到了兰州胜利机械租赁有限公司，被讯问人：裴金禄我又想到了我管理的中铁北京局和中铁电气化局施工公司。我先是通过百度搜索了“办证”之后就在网页上面弹出了一个页面上面有一个QQ号，我就加上了。加上之后我就将我的要求给他说了，要求他给我刻两个假的公章，一个是兰州胜利机械租赁有限公司合同专用章，另一个是中铁北京局集团有限公司合同专用章。我还要求他给我伪造了一张兰州胜利机械租赁有限公司的营业执照。
                回答：{"result":[{"startNodeType":"LawActor","entity":"裴金禄","endNodeType":"FictionalOrgan","property":"兰州胜利机械租赁有限公司","value":"冒充"},{"startNodeType":"LawActor","entity":"裴金禄","endNodeType":"Seal","property":"兰州胜利机械租赁有限公司合同专用章","value":"伪造"},{"startNodeType":"LawActor","entity":"裴金禄","endNodeType":"Seal","property":"中铁北京局集团有限公司合同专用章","value":"伪造"},{"startNodeType":"LawActor","entity":"裴金禄","endNodeType":"BusinessLicense","property":"兰州胜利机械租赁有限公司营业执照","value":"伪造"}]}。
                例子2:办案警官问：你为什么要在中卫市沙坡头区签订这些合同？裴金禄回答：因为我在这边上班，我在中卫市沙坡头区签订合同更能有说服力，这样才能更好地骗到钱。回答：{ "result": [] }。
                """;
    }


    @Override
    public String addNeo4j(List<String> ids) {
        List<TripleInfo> tripleInfos = tripleInfoService.listByIds(ids);
        int i = 0;
        for (TripleInfo tripleInfo : tripleInfos) {
            try {
                //开始节点
                String start = tripleInfo.getStartNode();
                // 首先看是否已经存在了,如果已经存在了,就不添加了
                CaseNode startNode = neo4jService.findOneByName(tripleInfo.getCaseId(), tripleInfo.getRecordId(), tripleInfo.getStartNodeType(), start, "1");
                if (startNode == null) {
                    startNode = new CaseNode(start, tripleInfo.getStartNodeType(), tripleInfo.getRecordSplitId(), tripleInfo.getRecordId(), tripleInfo.getCaseId(), "1");
                    CaseNode save = neo4jService.save(startNode);
                    startNode.setId(save.getId());
                }
                //结束节点
                String end = tripleInfo.getEndNode();
                CaseNode endNode = neo4jService.findOneByName(tripleInfo.getCaseId(), tripleInfo.getRecordId(), tripleInfo.getEndNodeType(), end, "1");
                if (endNode == null) {
                    endNode = new CaseNode(end, tripleInfo.getEndNodeType(), tripleInfo.getRecordSplitId(), tripleInfo.getRecordId(), tripleInfo.getCaseId(), "1");
                    CaseNode save = neo4jService.save(endNode);
                    endNode.setId(save.getId());
                }
                //关系
                Rel rel = new Rel(startNode.getId(), tripleInfo.getRelation(), endNode.getId());
                Rel relation = neo4jService.findRelation(rel);
                if (relation == null) {
                    neo4jService.saveRelation(rel);
                }
                tripleInfo.setAddNeo4j("1");
                boolean updateResult = tripleInfoService.updateById(tripleInfo);
                if (updateResult) {
                    i++;
                }
                // TODO 重复添加的OK了,删除的呢?
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
        return ("成功插入" + i + "条信息");
    }
}
